#include <Wire.h>
#include <Adafruit_Sensor.h>
#include <Adafruit_LSM303_U.h>
#include <MoleGraph.h>
MoleGraph moleGraph;
/* Assign a unique ID to this sensor at the same time */
Adafruit_LSM303_Mag_Unified mag = Adafruit_LSM303_Mag_Unified(12345);
Adafruit_LSM303_Accel_Unified accel = Adafruit_LSM303_Accel_Unified(54321);

//timer
unsigned long   previousTimer1 = 0;   
unsigned long   intervalTimer1 = 5; //20 ms = 50 Hz 

float aX, aY, aZ, aT;
float mX, mY, mZ, heading;
double pitch, roll;

/* for Low Pass Filter test
const float alpha = 0.5;
double fXg = 0;
double fYg = 0;
double fZg = 0;
*/

void UpdateGraphChannels(void)
{
  moleGraph.SetChannelValue(1, aT/10);
  moleGraph.SetChannelValue(2, heading);
  moleGraph.SetChannelValue(3, roll);
  moleGraph.SetChannelValue(4, -pitch);
}
void MeasurementStartedCallback(void)
{
  digitalWrite(13, HIGH);
}
void MeasurementStoppedCallback(void)
{
  digitalWrite(13, LOW);
}
void MeasurementPausedCallback(void)
{
  digitalWrite(13, LOW);
}
void MeasurementContinuedCallback(void)
{
  digitalWrite(13, HIGH);
}

void setup()
{
  moleGraph.Setup();
  moleGraph.SetSendingCallback(&UpdateGraphChannels);
  moleGraph.SetMeasurementStartedCallback(&MeasurementStartedCallback);
  moleGraph.SetMeasurementStoppedCallback(&MeasurementStoppedCallback);
  moleGraph.SetMeasurementPausedCallback(&MeasurementPausedCallback);
  moleGraph.SetMeasurementContinuedCallback(&MeasurementContinuedCallback);
  pinMode(13, OUTPUT);
  
  // Accelerometer initialization
  accel.begin();

  // Enable auto-gain + initialization
  mag.enableAutoRange(true);
  mag.begin();

  //write8(LSM303_ADDRESS_ACCEL, LSM303_REGISTER_ACCEL_CTRL_REG4_A, 0x00);  // 2G range (default)
  //write8(LSM303_ADDRESS_ACCEL, LSM303_REGISTER_ACCEL_CTRL_REG4_A, 0x10);  // 4G range
  //write8(LSM303_ADDRESS_ACCEL, LSM303_REGISTER_ACCEL_CTRL_REG4_A, 0x20);  // 8G range
  //write8(LSM303_ADDRESS_ACCEL, LSM303_REGISTER_ACCEL_CTRL_REG4_A, 0x30);  // 16G range 
}

void loop()
{
  //run my timer
  unsigned long currentTimer1 = millis();
  
  moleGraph.CheckInput();

  if(currentTimer1 - previousTimer1 > intervalTimer1) {
      
      sensors_event_t event1;
      sensors_event_t event2;
      
      /* Get a new sensor event */
      accel.getEvent(&event1);
      mag.getEvent(&event2);
    
      aX = event1.acceleration.x;
      aY = event1.acceleration.y;
      aZ = event1.acceleration.z;

      mX = event2.magnetic.x;
      mY = event2.magnetic.y;
      mZ = event2.magnetic.z;

      //calculate compass heading
      float Pi = 3.14159; 
      // Calculate the angle of the vector y,x
      heading = (atan2(mY, mX) * 180) / Pi;      
      // Normalize to 0-360
      if (heading < 0)
      {
        heading = 360 + heading;
      }

      //calculate total acceleration
      aT = sqrt(aX*aX + aY*aY + aZ*aZ);
      
      //calculate roll and pitch
      roll  = (atan2(aY, -aZ)*180.0)/M_PI;
      pitch = (atan2(aX, sqrt(aY*aY + aZ*aZ))*180.0)/M_PI;

      /*
      //Low Pass Filter test
      fXg = aX * alpha + (fXg * (1.0 - alpha));
      fYg = aY * alpha + (fYg * (1.0 - alpha));
      fZg = aZ * alpha + (fZg * (1.0 - alpha));
      //calculate roll and pitch from filtered values
      roll  = (atan2(-fYg, fZg)*180.0)/M_PI;
      pitch = (atan2(fXg, sqrt(fYg*fYg + fZg*fZg))*180.0)/M_PI;                            
      */
      
      previousTimer1 = currentTimer1;
  } 
}


